@@ -54,7 +54,8 @@ DataImpl :: DataImpl (bool unit_test, int cache_megs, DataIO * io):
   ProfilesImpl (*io),
   _cache (get_cache_path(), cache_megs),
   _unit_test (unit_test),
-  _data_io (io)
+  _data_io (io),
+  _descriptions_loaded (false)
 {
   rebuild_backend ();
 }
@@ -78,17 +79,26 @@ DataImpl :: rebuild_backend ()
     load_newsrc_files (*_data_io);
     load_group_xovers (*_data_io);
     load_group_permissions (*_data_io);
-    load_group_descriptions (*_data_io);
+
+    _descriptions.clear ();
+    _descriptions_loaded = false;
+
+    //load_group_descriptions (*_data_io);
     Log::add_info_va (_("Loaded data backend in %.1f seconds"), timer.get_seconds_elapsed());
   }
 }
 
 DataImpl :: ~DataImpl ()
+{
+  save_state ();
+}
+
+void
+DataImpl :: save_state ()
 {
   if (!_unit_test)
   {
-    debug ("data-impl dtor saving server, group, newsrc...");
-    save_group_xovers (*_data_io);
+    debug ("data-impl dtor saving newsrc...");
     save_newsrc_files (*_data_io);
   }
 }
