@@ -27,6 +27,7 @@ extern "C" {
   #include <glib.h>
   #include <glib/gi18n.h>
 }
+#include <pan/general/debug.h>
 #include <pan/general/file-util.h>
 #include <pan/general/line-reader.h>
 #include <pan/general/log.h>
@@ -198,9 +199,11 @@ namespace
 
     const std::string tmpfile (filename + ".tmp");
     if (ok) {
-      ::remove (filename.c_str());
-      ::rename (tmpfile.c_str(), filename.c_str());
-      ::chmod (filename.c_str(), 0600);
+      unlink (filename.c_str());
+      if (rename (tmpfile.c_str(), filename.c_str()))
+        std::cerr << LINE_ID << " ERROR renaming from [" << tmpfile << "] to [" << filename << "]: " << g_strerror(errno) << '\n';
+      if (chmod (filename.c_str(), 0600))
+        std::cerr << LINE_ID << " ERROR chmodding [" << filename << "]: " << g_strerror(errno) << '\n';
     } else {
       Log::add_err_va (_("Unable to save \"%s\" %s"), filename.c_str(), file::pan_strerror(my_errno));
       ::remove (tmpfile.c_str());
