@@ -170,17 +170,14 @@ DataImpl :: MyTree :: apply_filter (const const_nodes_v& candidates)
   // apply the filter to the whole tree
   if (_filter.empty())
     pass = candidates;
-  else {
-    foreach_const (const_nodes_v, candidates, it) {
-      if (!(*it)->_article)
-        continue;
-      else if (_data._article_filter.test_article (_data, _filter, _group, *(*it)->_article))
-        pass.push_back (*it);
-      else
-        fail.push_back (*it);
-    }
+  else foreach_const (const_nodes_v, candidates, it) {
+    if (!(*it)->_article)
+      continue;
+    else if (_data._article_filter.test_article (_data, _filter, _group, *(*it)->_article))
+      pass.push_back (*it);
+    else
+      fail.push_back (*it);
   }
-  g_assert (pass.size() + fail.size() == candidates.size());
   //std::cerr << LINE_ID << " " << pass.size() << " of "
   //          << (pass.size() + fail.size()) << " articles pass\n";
 
@@ -204,26 +201,30 @@ DataImpl :: MyTree :: apply_filter (const const_nodes_v& candidates)
       accumulate_descendants (d, *it);
     //std::cerr << LINE_ID << " expands into " << d.size() << " articles\n";
 
+    const_nodes_v fail2;
     pass.clear ();
     foreach_const (unique_nodes_t, d, it) {
       const Article * a ((*it)->_article);
       if (a->score > -9999 || _data._article_filter.test_article (_data, _filter, _group, *a))
         pass.push_back (*it); // pass is now sorted by mid because d was too
       else
-        fail.push_back (*it); // fail is now unsorted and may have duplicates
+        fail2.push_back (*it); // fail2 is sorted by mid because d was too
     }
 
-    // fail cleanup: remove duplicates
-    std::sort (fail.begin(), fail.end(), compare);
-    fail.erase (std::unique (fail.begin(), fail.end()), fail.end());
-
-    // fail cleanup: remove newly-passing articles
+    // fail cleanup: add fail2 and remove duplicates.
+    // both are sorted by mid, so set_union will do the job
     const_nodes_v tmp;
-    tmp.reserve (fail.size());
+    tmp.reserve (fail.size() + fail2.size());
+    std::set_union (fail.begin(), fail.end(),
+                    fail2.begin(), fail2.end(),
+                    inserter (tmp, tmp.begin()), compare);
+    fail.swap (tmp);
+	 
+    // fail cleanup: remove newly-passing articles
+    tmp.clear ();
     std::set_difference (fail.begin(), fail.end(),
                          pass.begin(), pass.end(),
-                         inserter (tmp, tmp.begin()),
-                         compare);
+                         inserter (tmp, tmp.begin()), compare);
     fail.swap (tmp);
     //std::cerr << LINE_ID << ' ' << pass.size() << " of "
     //          << (pass.size() + fail.size()) 
@@ -313,10 +314,12 @@ void
 DataImpl :: MyTree :: accumulate_descendants (unique_nodes_t& descendants,
                                               const ArticleNode* node) const
 {
-  if (node->_article)
-    descendants.insert (node);
-  foreach_const (ArticleNode::children_t, node->_children, it)
-    accumulate_descendants (descendants, *it);
+  // if this node has an article and wasn't already in `descendants',
+  // then add it and its children.
+
+  if (node->_article && descendants.insert(node).second)
+    foreach_const (ArticleNode::children_t, node->_children, it)
+      accumulate_descendants (descendants, *it);
 }
 
 void
@@ -469,9 +472,9 @@ DataImpl :: MyTree :: add_articles (const const_nodes_v& nodes_in)
                                  dend(descendants.end());
     while (nit!=nend && dit!=dend) {
       if (nit->second->_mid < (*dit)->_mid)
-        nit = _nodes.lower_bound ((*dit)->_mid);
+        ++nit;
       else if ((*dit)->_mid < nit->second->_mid)
-        dit = std::lower_bound (dit, dend, nit->second->_mid, compare);
+        ++dit;
       else {
         g_assert (nit->second->_mid == (*dit)->_mid);
         descend.push_back (TwoNodes (*dit, nit->second));
