@@ -167,6 +167,10 @@ namespace pan
       Data () {}
       virtual ~Data () {}
 
+    public:
+
+      virtual void save_state () = 0;
+
     public:
 
       virtual ArticleCache& get_cache () = 0; 
@@ -263,6 +267,8 @@ namespace pan
         std::string description;
         /** y, n, or m for posting, no posting, moderated */
         char permission;
+
+        NewGroup(): permission('?') {}
       };
 
       /**
