@@ -275,7 +275,8 @@ namespace
     if (gtk_text_iter_has_tag (&iter, url_tag))
     {
       GtkTextIter begin(iter), end(iter);
-      gtk_text_iter_backward_to_tag_toggle (&begin, NULL);
+      if (!gtk_text_iter_begins_tag (&begin, url_tag))
+        gtk_text_iter_backward_to_tag_toggle (&begin, NULL);
       gtk_text_iter_forward_to_tag_toggle (&end, NULL);
       retval = gtk_text_iter_get_text (&begin, &end);
     }
@@ -380,12 +381,30 @@ namespace
           if (!gtk_text_iter_begins_tag (&iter, pix_tag))
             gtk_text_iter_backward_to_tag_toggle (&iter, pix_tag);
           g_assert (gtk_text_iter_begins_tag (&iter, pix_tag));
+
+          // percent_x,percent_y reflect where the user clicked in the picture
+          GdkRectangle rec;
+          gtk_text_view_get_iter_location (GTK_TEXT_VIEW(w), &iter, &rec);
+          int buf_x, buf_y;
+          gtk_text_view_window_to_buffer_coords (GTK_TEXT_VIEW(w), GTK_TEXT_WINDOW_WIDGET,
+                                                 (gint)event->x, (gint)event->y, &buf_x, &buf_y);
+          const double percent_x = (buf_x - rec.x) / (double)rec.width;
+          const double percent_y = (buf_y - rec.y) / (double)rec.height;
+
+           // resize the picture and refresh `iter'
           const int offset (gtk_text_iter_get_offset (&iter));
           GtkTextBuffer * buf (gtk_text_view_get_buffer (GTK_TEXT_VIEW(w)));
           const bool fullsize (toggle_fullsize_flag (buf));
           resize_picture_at_iter (buf, &iter, fullsize, &w->allocation, pix_tag);
           gtk_text_iter_set_offset (&iter, offset);
           set_cursor_from_iter (event->window, w, &iter);
+
+          // x2,y2 are to position percent_x,percent_y in the middle of the window.
+          GtkTextMark * mark = gtk_text_buffer_create_mark (buf, NULL, &iter, true);
+          const double x2 = CLAMP ((percent_x + (percent_x - 0.5)), 0.0, 1.0);
+          const double y2 = CLAMP ((percent_y + (percent_y - 0.5)), 0.0, 1.0);
+          gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW(w), mark, 0.0, true, x2, y2);
+          gtk_text_buffer_delete_mark (buf, mark);
         }
       }
     }
@@ -702,30 +721,56 @@ namespace
       gtk_text_buffer_apply_tag_by_name (buffer, last_quote_tag.c_str(), &mark_start, &mark_end);
     }
 
+    // apply markup
     const StringView v_all (body);
-    if (do_markup) {
+    if (do_markup)
+    {
+      bool is_patch = false;
+      std::set<const char*> already_processed;
       StringView v(body), line;
-      while (v.pop_token (line, '\n')) {
-        const char * prev (0);
-        for (;;) {
+      while (v.pop_token (line, '\n'))
+      {
+        // if we detect that this message contains a patch,
+        // turn off markup for the rest of the article.
+        if (!is_patch)
+          is_patch = (line.strstr("--- ")==line.str) || (line.strstr("@@ ")==line.str);
+        if (is_patch)
+          continue;
+
+        for (;;)
+        {
+          // find the first markup character.
+          // if we've already used it,
+          // or if it was preceeded by something not a space or punctuation,
+          // then keep looking.
           const char * b (line.strpbrk ("_*/"));
-          if (!b) break;
-          // if it's a new character, and (the first char in the line OR previous char is space/punctuation)
-          if (b != prev && (b==&line.front() || (isspace(b[-1] || ispunct(b[-1]))))) {
-            const StringView lineleft (line.substr (b+1, 0));
-            const char * e = lineleft.strchr (*b);
-            // if there's a closing character and (it's the last character OR the following character is space/punctuation)
-            if (e && (e==&lineleft.back() || (isspace(e[1]) || ispunct(e[1])))) {
-              const char * type (0);
-              switch (*b) {
-                case '*': type = "bold"; break;
-                case '_': type = "underline"; break;
-                case '/': type = "italic"; break;
-              }
-              set_section_tag (buffer, &start, v_all, StringView(b,e+1), type, ADD);
-            }
-            prev = e;
+          if (!b)
+            break;
+          if (already_processed.count(b) ||
+              (b!=&line.front() && !isspace(b[-1]) && !ispunct(b[-1]))) {
+            line.eat_chars (b+1-line.str);
+            continue;
           }
+
+          // find the ending corresponding markup character.
+          // if it was followed by something not a space or punctuation, keep looking.
+          const char * e = b;
+          while ((e = line.strchr (*b, 1+(e-line.str)))) {
+            if (e==&line.back() || isspace(e[1]) || ispunct(e[1]) || strchr("_*/", e[1]))
+              break;
+          }
+
+          if (e) {
+            already_processed.insert (e);
+            const char * type (0);
+            switch (*b) {
+              case '*': type = "bold"; break;
+              case '_': type = "underline"; break;
+              case '/': type = "italic"; break;
+            }
+            set_section_tag (buffer, &start, v_all, StringView(b,e+1), type, ADD);
+          }
+
           line.eat_chars (b+1-line.str);
         }
       }
@@ -1053,7 +1098,7 @@ BodyPane :: set_text_from_message (GMimeMessage * message)
   // if there was a picture, scroll to it.
   // otherwise scroll to the top of the body.
   GtkTextMark * mark = gtk_text_buffer_get_mark (_buffer, FIRST_PICTURE);
-  if (mark)
+  if (mark && _prefs.get_flag ("focus-on-image", true))
     gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW(_text), mark, 0.0, true, 0.0, 0.0);
   else {
     GtkTextIter iter;
@@ -1369,20 +1414,29 @@ BodyPane :: read_more_or_less (bool more)
 
 namespace
 {
-  // "Re: RE: Re: Re: foo" --> "Re: foo"
-  StringView normalize_subject_re (const StringView& v_in)
+  // (1) strip redundant leading Re: and RE:
+  // (2) ensure the remaining Re: has a lowercase e
+
+  std::string normalize_subject_re (const StringView& v_in)
   {
-    StringView ret, v(v_in);
+    StringView v(v_in), prev(v_in);
     for (;;) {
       v.ltrim ();
-      ret = v;
-      if (v.strstr("Re:") == v.str)
-        v.eat_chars (3);
-      else if (v.strstr("RE:") == v.str)
-        v.eat_chars (3);
+      StringView tmp (v);
+      if (tmp.strstr("Re:") == tmp.str)
+        tmp.eat_chars (3);
+      else if (v.strstr("RE:") == tmp.str)
+        tmp.eat_chars (3);
       else
         break;
+      prev = v;
+      v = tmp;
     }
+
+    std::string ret (prev.str, prev.len);
+    if (!ret.find("RE:")) // force lowercase 'e'
+      ret.replace (0, 3, "Re:");
+
     return ret;
   }
 
@@ -1469,19 +1523,16 @@ BodyPane :: create_followup_or_reply (bool is_reply)
     }
 
     // Subject:
-    StringView v = g_mime_message_get_subject (_message);
+    StringView v = g_mime_message_get_header (_message, "Subject");
     std::string h = header_to_utf8 (v, message_charset, group_charset);
-    v = normalize_subject_re (h);
-    std::string val (v.str, v.len);
-    if (!val.find("RE:") || !val.find ("Re:"))
-      val.replace (0, 3, "Re:"); // be polite & force lowercase 'e'
-    else
-      val.insert (0, "Re: "); // no Re: -- add one.
+    std::string val (normalize_subject_re (h));
+    if (val.find ("Re:") != 0) // add "Re: " if we don't have one
+      val.insert (0, "Re: ");
     g_mime_message_set_subject (msg, val.c_str());
 
     // attribution lines
 
-    const char * cpch = g_mime_message_get_sender (_message);
+    const char * cpch = g_mime_message_get_header (_message, "From");
     h = header_to_utf8 (cpch, message_charset, group_charset);
     g_mime_message_add_header (msg, "X-Draft-Attribution-Author", h.c_str());
 
