@@ -21,7 +21,6 @@
 #define _GroupPane_h_
 
 #include <gtk/gtk.h>
-#include <pan/general/map-vector.h>
 #include <pan/general/quark.h>
 #include <pan/data/data.h>
 #include <pan/gui/action-manager.h>
@@ -53,6 +52,7 @@ namespace pan
       GroupPane (ActionManager&, Data&, Prefs&);
       ~GroupPane ();
       GtkWidget* root () { return _root; }
+      GtkWidget* get_default_focus_widget() { return _tree_view; }
       GtkWidget* create_filter_entry ();
       void set_name_collapse (bool);
       Quark get_selection () const;
@@ -67,8 +67,6 @@ namespace pan
       GtkWidget * _root;
       GtkWidget * _tree_view;
       PanTreeStore * _tree_store;
-      typedef Loki::AssocVector<Quark,MyRow*> group_rows_t;
-      group_rows_t _group_rows;
 
     private:
       static void do_popup_menu (GtkWidget*, GdkEventButton*, gpointer);
